#include QMK_KEYBOARD_H

enum custom_keycodes {
    CTL_SPC = SAFE_RANGE,
};

bool get_custom_auto_shifted_key(uint16_t keycode, keyrecord_t *record) {
        if(IS_RETRO(keycode))
            return true;
        return false;
}

bool process_record_user(uint16_t keycode, keyrecord_t *record) {
    static uint16_t pressed_timer;
    switch (keycode) {
        case CTL_SPC:
            if (record->event.pressed) {
                pressed_timer = timer_read();
                register_code(KC_LGUI);
            } else {
                unregister_code(KC_LGUI);
                if (timer_elapsed(pressed_timer) < TAPPING_TERM) {
                    register_code(KC_LCTL);
                    SEND_STRING(" ");
                    unregister_code(KC_LCTL);
                }
            }
            return false;
    }
    return true;
}

uint16_t get_tapping_term(uint16_t keycode, keyrecord_t *record) {
    switch (keycode) {
        case MT(MOD_LSFT, KC_A):
        case LT(4, KC_R):
        case LT(2, KC_S):
        case LT(1, KC_T):
        case LT(3, KC_G):
        case LT(3, KC_M):
        case LT(1, KC_N):
        case LT(2, KC_E):
        case LT(4, KC_I):
        case MT(MOD_RSFT, KC_O):
            return TAPPING_TERM + 75;
        default:
            return TAPPING_TERM;
    }
}

const uint16_t PROGMEM combo1[] = {KC_SPC, KC_S, COMBO_END};
const uint16_t PROGMEM combo2[] = {CTL_SPC, KC_S, COMBO_END};
const uint16_t PROGMEM combo3[] = {KC_SPC, KC_T, COMBO_END};
const uint16_t PROGMEM combo4[] = {CTL_SPC, KC_T, COMBO_END};
const uint16_t PROGMEM combo5[] = {KC_SPC, KC_R, COMBO_END};
combo_t            key_combos[] = {
    COMBO(combo1, OSL(4)),
    COMBO(combo2, OSL(2)),
    COMBO(combo3, OSL(2)),
    COMBO(combo4, OSL(4)),
    COMBO(combo5, OSL(3)),
};

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
	[0] = LAYOUT_split_3x6_3(
            LALT_T(KC_TAB),   MT(MOD_LALT,KC_Q), KC_W, KC_F, KC_P, KC_B,              KC_J, KC_L, KC_U,    KC_Y,   MT(MOD_RALT,KC_SCLN), KC_BSPC,
            LCTL_T(KC_ESC), MT(MOD_LSFT,KC_A), LT(4,KC_R), LT(2,KC_S), LT(1,KC_T), LT(3,KC_G), LT(3,KC_M), LT(1,KC_N), LT(2,KC_E), LT(4,KC_I), MT(MOD_RSFT,KC_O), RCTL_T(KC_QUOT),
            SC_LSPO, KC_Z,  KC_X, KC_C, KC_D, KC_V,                    KC_K, KC_H, KC_COMM, KC_DOT, KC_SLSH, SC_RSPC,
                         KC_BSPC, KC_SPC, CTL_SPC,                      RALT_T(KC_TAB), KC_ENT, KC_BSPC
    ),

	[1] = LAYOUT_split_3x6_3(
            LALT_T(KC_TAB), KC_EXLM, KC_AT, KC_HASH, KC_DLR, KC_PERC,    KC_CIRC, KC_AMPR, KC_ASTR, KC_LPRN, KC_RPRN, KC_BSPC,
            LCTL_T(KC_ESC), KC_NO, KC_AMPR, KC_ASTR, KC_CIRC, CW_TOGG,   KC_MINS, KC_EQL, KC_LBRC, KC_RBRC, KC_BSLS, KC_GRV,
            KC_LSFT, KC_NO, KC_NO, KC_NO, C(KC_L), KC_NO,                KC_UNDS, KC_PLUS, KC_LCBR, KC_RCBR, KC_PIPE, KC_TILD,
                                     TO(0), KC_SPC, CTL_SPC,            RALT_T(KC_TAB), LT(3,KC_ENT), TO(1)
    ),
	[2] = LAYOUT_split_3x6_3(

            LALT_T(KC_TAB), KC_F1, KC_F2, KC_F3, KC_F4, KC_F5,           KC_CIRC, KC_7, KC_8, KC_9, KC_0, KC_BSPC,
            LCTL_T(KC_ESC), KC_F6, KC_F7, KC_F8, KC_F9, KC_F10,        KC_COMM, KC_4, KC_5, KC_6, KC_MINS, KC_EQL, SC_LSPO,
            KC_NO, KC_NO, KC_DEL, KC_F11, KC_F12,                      KC_DOT, KC_1, KC_2, KC_3, KC_SLSH, KC_ASTR,
                                 TO(0), KC_SPC, CTL_SPC,                KC_RSFT, LT(3,KC_ENT), TO(2)
    ),

	[3] = LAYOUT_split_3x6_3(
            KC_MUTE, KC_MPRV, KC_MNXT, KC_VOLD, KC_VOLU, KC_MPLY,      KC_MS_D, KC_MS_R, KC_WH_R, KC_NO, KC_NO, KC_ACL0,
            LCTL_T(KC_ESC), KC_NO, KC_NO, KC_BTN3, KC_BTN1, KC_BTN2,   KC_NO, KC_WH_U, KC_WH_D, KC_NO, OSL(4), KC_ACL1,
            KC_LSFT, KC_NO, KC_NO, KC_NO, KC_NO, KC_NO,                KC_MS_U, KC_MS_L, KC_WH_L, KC_NO, KC_NO, KC_ACL2,
                                              TO(0), KC_SPC, CTL_SPC,   KC_NO, KC_NO, TO(3)
    ),

	[4] = LAYOUT_split_3x6_3(
            RGB_TOG, RGB_M_B, RGB_M_P, RGB_VAD, RGB_VAI, KC_NO,      KC_DOWN, KC_RGHT, KC_PGUP, KC_HOME, AS_TOGG, TO(0),
            LCTL_T(KC_ESC), RGB_M_R, RGB_M_SW, RGB_HUD, RGB_HUI, RGB_M_TW,    KC_NO, KC_NO, KC_NO, KC_NO, KC_PSCR, RCTL_T(KC_QUOT),
            KC_LSFT, RGB_M_G, KC_NO, RGB_SAD, RGB_SAI, KC_NO,     KC_UP, KC_LEFT, KC_PGDN, KC_END, KC_NO, KC_NO,
                                         TO(0), KC_SPC, CTL_SPC,        KC_LGUI, KC_ENT, TO(4)
    ),
};

